/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FAULTD_SYSTEMD_DBUS_H
#define FAULTD_SYSTEMD_DBUS_H

#include <systemd/sd-bus.h>

#define SYSTEMD_SERVICE "org.freedesktop.systemd1"

#define SYSTEMD_OBJ "/org/freedesktop/systemd1"

#define SYSTEMD_MANAGER_INTERFACE "org.freedesktop.systemd1.Manager"
#define SYSTEMD_UNIT_INTERFACE "org.freedesktop.systemd1.Unit"

#define SYSTEMD_PRIVATE_BUS "unix:path=/run/systemd/private"
#define SYSTEMD_KERNEL_BUS "kernel:path=/sys/fs/kdbus/0-system/bus"

int faultd_dbus_call(char *service, char *obj, char *interface, char *method,
					 sd_bus_error *error, sd_bus_message **out_msg,
					 char *args, ...);

#define faultd_dbus_call_simple(SRV, OBJ, INTF, METHOD, ARGS, ...)	\
	faultd_dbus_call(SRV, OBJ, INTF, METHOD, NULL, NULL, ARGS, ##__VA_ARGS__)

#define faultd_dbus_call_systemd_simple(OBJ, INTF, METHOD, ARGS, ...) \
	faultd_dbus_call_simple(SYSTEMD_SERVICE, OBJ, INTF, METHOD,		  \
							ARGS, ##__VA_ARGS__)

int faultd_acquire_systemd_bus(sd_bus **bus);

#endif /* FAULTD_SYSTEMD_DBUS_H */
