#include "resource_violation_event.h"
#include "helpers.h"

#define TESTED_MOD resource_violation_event

DECLARE_TEST_ENTRY_POINT(TESTED_MOD, rv_event_serialize, void);

static struct faultd_event *alloc_event()
{
	struct resource_violation_event *rv_ev;

	rv_ev = calloc(1, sizeof(*rv_ev));
	assert_non_null(rv_ev);

	fill_event(&rv_ev->event);
	fill_service(&rv_ev->service);
	rv_ev->detection_time = generate_time_t();
	rv_ev->pid = generate_int();
	rv_ev->resource_type = generate_int();

	return &rv_ev->event;
}

static void generate_test_attrs(struct serialize_test_data *td)
{
	struct resource_violation_event *rv_ev = to_resource_violation_event(td->ev);
	struct serialized_element service_data[] = {
		S_ELEMENT_SERVICE(rv_ev->service)
	};

	struct serialized_element serialized_attrs[] = {
		S_ELEMENT_EVENT(rv_ev->event),
		S_ELEMENT_OBJ(SYSD_SERVICE, NULL,
					  service_data, ARRAY_SIZE(service_data)),
		S_ELEMENT(RV_EV_DTIME, TYPE_TIME_T, &rv_ev->detection_time),
		S_ELEMENT(RV_EV_RES_TYPE, TYPE_INT, &rv_ev->resource_type),
		S_ELEMENT(RV_EV_PID, TYPE_INT, &rv_ev->pid)
	};

	td->test_data = serialized_elems_dup(serialized_attrs,
										 ARRAY_SIZE(serialized_attrs));
	td->size = ARRAY_SIZE(serialized_attrs);
}

static int setup(void **state)
{
	return setup_event_serialization(state, alloc_event, generate_test_attrs,
									 (serialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, rv_event_serialize));
}

static int teardown(void **state)
{
	struct serialize_test_data *td = *state;
	struct resource_violation_event *rv_ev = to_resource_violation_event(td->ev);

	free_event(&rv_ev->event);
	free_service(&rv_ev->service);
	free(rv_ev);
	serialized_elems_free(td->test_data, td->size);
	free(td);
	return 0;
}

FAULTD_TEST_GROUP(
	FAULTD_TEST_CASE_NST("rv_ev_serialize_test", test_event_serialization,
						 setup, teardown)
)
