#include "service_failed_event.h"
#include "helpers.h"

#define TESTED_MOD service_failed_event

DECLARE_TEST_ENTRY_POINT(TESTED_MOD, sf_event_serialize, void);

static struct faultd_event *alloc_event()
{
	struct service_failed_event *sf_ev;

	sf_ev = calloc(1, sizeof(*sf_ev));
	assert_non_null(sf_ev);

	fill_event(&sf_ev->event);
	fill_service(&sf_ev->service);
	sf_ev->detection_time = generate_time_t();

	return &sf_ev->event;
}

static void generate_test_attrs(struct serialize_test_data *td)
{
	struct service_failed_event *sf_ev = to_service_failed_event(td->ev);
	struct serialized_element service_data[] = {
		S_ELEMENT_SERVICE(sf_ev->service)
	};

	struct serialized_element serialized_attrs[] = {
		S_ELEMENT_EVENT(sf_ev->event),
		S_ELEMENT_OBJ(SYSD_SERVICE, NULL,
					  service_data, ARRAY_SIZE(service_data)),
		S_ELEMENT(SF_EV_DTIME, TYPE_TIME_T, &sf_ev->detection_time)
	};

	td->test_data = serialized_elems_dup(serialized_attrs,
										 ARRAY_SIZE(serialized_attrs));
	td->size = ARRAY_SIZE(serialized_attrs);
}

static int setup(void **state)
{
	return setup_event_serialization(state, alloc_event, generate_test_attrs,
									 (serialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, sf_event_serialize));
}

static int teardown(void **state)
{
	struct serialize_test_data *td = *state;
	struct service_failed_event *sf_ev = to_service_failed_event(td->ev);

	free_event(&sf_ev->event);
	free_service(&sf_ev->service);
	free(sf_ev);
	serialized_elems_free(td->test_data, td->size);
	free(td);
	return 0;
}

FAULTD_TEST_GROUP(
	FAULTD_TEST_CASE_NST("sf_ev_serialize_test", test_event_serialization,
						 setup, teardown)
)
